#!/usr/bin/ruby

require 'jcode'

class Node
  #der Pointer auf den Nächsten in der Suffixkette
  attr_accessor :suf
  #der Name des Knotens (eine Nummer)
  attr_reader :name
  def initialize(name)
    @name=name
    @pointer = {}
  end
  #Setzt eine Kante mit dem übergang _with_ zum Knoten _to_ mit dem Typen _type_
  def edge(with,to,type)
    @pointer[with] = [to,type]
  end
  #Liefert den [Knoten,Typ], der über die Kante _with_ erreichbar ist oder nil.
  def son(with)
    @pointer[with]
  end
  #ruft den übergebenen Block mit den Parametern _with_,_Knoten_,_Typ_ für jede Kante von diesem Knoten auf.
  def each 
    @pointer.each_pair do |with,to|
      yield with,to[0],to[1]
    end
  end
  #Ruft den Block rekursiv mit jedem von diesem Knoten erreichbaren Knoten auf.
  def traverse(&block)
    yield self
    each do |with,to,type|
      to.traverse(&block)
    end
  end
end


class SubwordGraph
  def initialize(string)
    @autonum=0
    @root=Node.new(nextnum)
    create(string)
  end

  #gibt den Graphen in einem Stream im dot Format aus
  def print(io)
    io.puts "Digraph myTrie { //DOTCODE
rankdir=LR; //DOTCODE
ordering=out; //DOTCODE
splines=true; //DOTCODE
"
    seen=[]
    @root.traverse do |node|
      unless seen.include?(node)
	io.puts "#{node.name}[peripheries=1]; //DOTCODE\n"
	io.puts "#{node.name}->#{node.suf.name}[style=dashed]; //DOTCODE\n" if node.suf
	
	node.each do |with,to,type|
	  if type==:solid
	    io.puts "#{node.name}->#{to.name}[style=bold,weight=2.0,label=\"#{with}\"]; //DOTCODE\n"
	  else
	    io.puts "#{node.name}->#{to.name}[style=solid,label=\"#{with}\"]; //DOTCODE\n"
	  end
	end
	seen << node
      end
    end
    io.puts "}; //DOTCODE\n"
  end

private
  #erzeugt den Subgraphen
  def create(string)
    sink=@root

    string.each_char do |a|
      newsink=Node.new(nextnum)
      sink.edge(a,newsink,:solid)
      w=sink.suf

      while w != nil && w.son(a) == nil
	w.edge(a,newsink,:non_solid)
	w=w.suf
      end

      if w==nil 
	newsink.suf=@root
      else
	v,type=w.son(a)
	if type==:solid
	  newsink.suf=v
	else
	  newnode=Node.new(nextnum)
	  #copy the outgoing edges
	  v.each do |with,to,_type|
	    newnode.edge(with,to,:non_solid)
	  end
	  w.edge(a,newnode,:solid)
	  newsink.suf=newnode
	  newnode.suf=v.suf
	  v.suf=newnode
	  w=w.suf

	  while w != nil && w.son(a) == [v,:non_solid]
	    w.edge(a,newnode,:non_solid)
	    w=w.suf
	  end
	end
      end
      sink = newsink
    end # each_char
  end
  #Gibt eine eindeutige Nummer zurück
  def nextnum
    @autonum +=1
  end
end



wrd=""

ARGV[0].each_char do |a|
  wrd+=a
  sg=SubwordGraph.new(wrd)
  File.open("tmp_sg.dot",'w') do |io|
    sg.print(io)
  end
  system("cat tmp_sg.dot | grep \"DOTCODE\" | dot -Tpng -o #{wrd}.png")
end
